<?php
/*
See yak-for-wordpress.php for information and license terms
*/

/**
 * Data class describing items in an order.
 */
class YakItem {

    var $id_type;
    var $id;
    var $cat_id;
    var $quantity;
    var $name;
    var $item_weight;
    var $special_option;
    
    var $price;
    var $discount;

    /**
     * Constructor
     *
     * @param $id_type post or page type
     * @param $id the product id (i.e. post id or page id)
     * @param $cat_id category for the product type
     * @param $quantity amount to purchase
     */
    function YakItem($id_type, $id, $cat_id, $quantity) {
        $this->id_type = $id_type;
        $this->id = $id;
        $this->cat_id = $cat_id;
        $this->quantity = $quantity;
    }
    
    function get_total_weight() {
        return $this->quantity * $this->item_weight;
    }
    
    function get_discount_total() {
        return $this->quantity * $this->get_discount_price();
    }
    
    function get_discount_price() {
        return $this->price - $this->discount;
    }
    
    function get_total() {
        return $this->quantity * $this->price;
    }
}


/**
 * Data class describing an order.
 */
class YakOrder {
    var $id;
    var $num;
    var $time;
    var $address;
    var $billing_address;
    var $payment_type;
    var $funds_received;
    var $shipping_cost;
    var $total;
    var $status;
    
    var $items;
    var $log;
    var $meta;
    
    /**
     * Constructor
     *
     * @param $id the id of the order
     * @param $num the external order number (displayed to a customer)
     * @param $time the time of the order
     * @param $address the shipping address
     * @param $billing_address address to send the bill to
     * @param $payment_type method of paying (Credit Card, Deposit, Cheque, etc)
     * @param $funds_received payment received so far
     * @param $shipping_cost cost of shipping the order
     * @param $total total cost of the order
     * @param $status current status of the order
     */
    function YakOrder($id, $num, $time, $address, $country_code, $billing_address, $billing_country_code, $payment_type, $funds_received, 
                      $shipping_cost, $total, $status) {
        $this->id = $id;
        $this->num = $num;
        $this->time = $time;
        $this->address = $address;
        $this->billing_address = $billing_address;
        $this->payment_type = $payment_type;
        $this->funds_received = $funds_received;
        $this->shipping_cost = $shipping_cost;
        $this->total = $total;
        $this->status = $status;
    }
}


/**
 * Class describing a product in YAK
 */
class YakProduct {
    
    var $post_title;
    var $id;
    var $ID;
    var $status;
    var $title;
    var $price;
    var $content;
    
    var $types;
    
    /**
     * Constructor
     *
     * @param $id the id of the post for the product
     * @param $post_title the title of the post (hence the name of the product)
     * @param $status the status of the post/page
     * @param $title alternative title for the product
     * @param $price price of the product
     */
    function YakProduct($id, $post_title, $status, $title, $price) {
        $this->id = $id;
        $this->ID = $id;
        $this->post_title = $post_title;
        $this->status = $status;
        $this->title = $title;
        $this->price = $price;
        
        $this->types = array();
    }
}


/**
 * Class describing a product type (category)
 */
class YakProductType {
    var $post_id;
    var $cat_id;
    var $name;
    var $sku;
    var $qty;
    var $override_price;
    var $weight;
    var $dl_file;
    
    /** 
     * Constructor
     *
     * @param $post_id reference to the product id
     * @param $cat_id the id of the category for this type of product
     * @param $name the name of the category/product type
     * @param $qty the quantity available of this type
     * @param $weight the weight of this type
     * @param $dl_file the download file if this is a downloadable product
     */
    function YakProductType($post_id, $cat_id, $name, $sku, $qty, $override_price, $weight, $dl_file) {
        $this->post_id = $post_id;
        $this->cat_id = $cat_id;
        $this->name = $name;
        $this->sku = $sku;
        $this->qty = $qty;
        $this->override_price = $override_price;
        $this->weight = $weight;
        $this->dl_file = $dl_file;
    }
}


/**
 * Class representing a summary total (for the reports page)
 */
class YakTotal {
    var $desc;
    var $secondary_desc;
    var $total;
    var $secondary_total;
    
    function YakTotal($desc, $total, $secondary_total = 0, $secondary_desc = '') {
        $this->desc = $desc;
        $this->total = $total;
        $this->secondary_total = $secondary_total;
        $this->secondary_desc = $secondary_desc;
    }
}


/**
 * Class representing a promotion (temporary reduction is price or shipping)
 */
class YakPromotion {
    var $promo_id;
    var $code;
    var $promo_type;
    var $value;
    var $description;
    var $discount;
    var $users;
    var $expiry_date;
    
    function YakPromotion($promo_id, $code, $promo_type, $value, $description, $expiry_date, $users) {
        $this->promo_id = $promo_id;
        $this->code = $code;
        $this->promo_type = $promo_type;
        $this->value = $value;
        $this->description = $description;
        $this->expiry_date = $expiry_date;
        $this->users = $users;
    }
    
    function get_users_string() {
        if (!empty($this->users)) {
            return implode(',', $this->users);
        }
        else {
            return '';
        }
    }
}


/**
 * Class representing a shipping or billing address
 */
class YakAddress {
    var $email;
    var $recipient;
    var $phone;
    var $addr1;
    var $addr2;
    var $suburb;
    var $city;
    var $region;
    var $state;
    var $postcode;
    var $country;
    
    function YakAddress($email, $recipient, $phone, $addr1, $addr2, $suburb, $city, $region, $state, $postcode, $country) {
        $this->email = $email;
        $this->recipient = stripslashes($recipient);
        $this->phone = $phone;
        $this->addr1 = stripslashes($addr1);
        $this->addr2 = stripslashes($addr2);
        $this->suburb = stripslashes($suburb);
        $this->city = stripslashes($city);
        $this->region = stripslashes($region);
        $this->state = $state;
        $this->postcode = $postcode;
        $this->country = $country;
    }
    
    function get_members() {
        $arr = array();
        $arr['email'] = $this->email;
        $arr['recipient'] = $this->recipient;
        $arr['phone'] = $this->phone;
        $arr['addr1'] = $this->addr1;
        $arr['addr2'] = $this->addr2;
        $arr['suburb'] = $this->suburb;
        $arr['city'] = $this->city;
        $arr['region'] = $this->region;
        $arr['state'] = $this->state;
        $arr['postcode'] = $this->postcode;
        $arr['country'] = $this->country;
        return $arr;
    }
    
    function as_string() {
        $args = func_get_args();
        $rtn = '';
        foreach ($this->get_members() as $key => $val) {
            if (!empty($val) && !in_array($key, $args)) {
                $rtn .= $val . "\n";
            }
        }
        return $rtn;
    }
    
    function get_state_or_region() {
        if (!empty($this->state)) {
            return $this->state;
        }
        else {
            return $this->region;
        }
    }
    
    function get_first_name() {
        $arr = split("[\n\r\t ]+", $this->recipient);
        return $arr[0];
    }
    
    function get_last_name() {
        $arr = split("[\n\r\t ]+", $this->recipient);
        return $arr[1];
    }
}
   

/**
 * Class used to hold a parameter array and position elements
 */   
class YakParams {
    var $params;
    var $pos1;
    var $pos2;
    
    function YakParams($pos1, $pos2) {
        $this->pos1 = $pos1;
        $this->pos2 = $pos2;
    }
}

class YakOrderWidget {    
    function widget($args) {
        if (isset($_SESSION['current_order_value'])) {
            echo $args['before_widget'];
            echo $args['before_title'] . __('Your Shopping Basket', 'yak') . $args['after_title'];
            echo 'Items in basket: ' . $_SESSION['current_order_items'] . '<br />';
            echo 'Subtotal: ' . yak_format_money($_SESSION['current_order_value'], true);
            echo $args['after_widget'];
        }
    }
    
    function register() {
        register_sidebar_widget('Yak Order Panel', array('YakOrderWidget', 'widget'));
    }
}
?>